local betObjectId = 2114 -- id of object where bet(money) has to be moved on -- https://imgur.com/a/wxzK2u8
local betMoneyId = 3043 -- id of money to bet (3043 - cc) -- https://imgur.com/a/HZ7JWTy
local defaultBet = 50-- 50 cc -- https://imgur.com/a/HZ7JWTy
local possibleBets = { 5, 10, 25, 50 }
local exchangeIds = { 3040, 11587, 8192 } -- money to exchange when out of cc (3040 - gold nugget, 11587 - dark coin)

local switchObjectIds = { 2772, 2773 } -- https://imgur.com/a/aCuW17s -- https://imgur.com/a/vcBSdJM
local standTileIds = { 564, 10145, 10146 }

local betObjectPos = nil
local switchObjectPos = nil

function getBetObjectSwitchPos(pos)
  local tilesAround =
  {
    { x = pos.x - 1, y = pos.y - 1, z = pos.z }, -- north west
    { x = pos.x,     y = pos.y - 1, z = pos.z }, -- north
    { x = pos.x + 1, y = pos.y - 1, z = pos.z }, -- north east
    
    { x = pos.x - 1, y = pos.y, z = pos.z }, -- west
    { x = pos.x + 1, y = pos.y, z = pos.z }, -- east
    
    { x = pos.x - 1, y = pos.y + 1, z = pos.z }, -- south west
    { x = pos.x,     y = pos.y + 1, z = pos.z }, -- south
    { x = pos.x + 1, y = pos.y + 1, z = pos.z } -- south east
  }
  -- check tiles around player for betObjectPos and switchObjectPos
  for _, tileAround in pairs(tilesAround) do
    local tile = g_map.getTile(tileAround)
    for _, item in pairs(tile:getItems()) do
      if item:getId() == betObjectId then
        betObjectPos = tile:getPosition()
      elseif table.find(switchObjectIds, item:getId()) then
        switchObjectPos = tile:getPosition()
      end
    end
  end
end

function hasCC() -- look in all opened bps for crystal coins
  for _, container in pairs(getContainers()) do
    for _, item in pairs(container:getItems()) do
      if item:getId() == betMoneyId and item:getCount() >= defaultBet then
        return item
      end
    end
  end
  return nil
end

function moveWinToBp(pos)
  local winTiles =
  {
    { x = pos.x - 1, y = pos.y - 1, z = pos.z }, -- north west
    { x = pos.x,     y = pos.y - 1, z = pos.z }, -- north
    { x = pos.x + 1, y = pos.y - 1, z = pos.z }, -- north east
    
    { x = pos.x - 1, y = pos.y, z = pos.z }, -- west
    { x = pos.x + 1, y = pos.y, z = pos.z }, -- east
    
    { x = pos.x - 1, y = pos.y + 1, z = pos.z }, -- south west
    { x = pos.x,     y = pos.y + 1, z = pos.z }, -- south
    { x = pos.x + 1, y = pos.y + 1, z = pos.z } -- south east
  }
  -- check if player won and move prize to bp
  for _, winTile in pairs(winTiles) do
    local tile = g_map.getTile(winTile)
    if tile then
      local topLookThing = tile:getTopLookThing()
      if topLookThing and topLookThing:getId() == betMoneyId then
        g_game.move(topLookThing, {x=65535, y=3, z=0}, topLookThing:getCount())
      end
    end
  end
end

macro(100, "Casino", function()
  local playerPos = pos()
  if g_map.getTile(playerPos):getGround():getId() == standTileId then return end
  getBetObjectSwitchPos(playerPos)
  
  if betObjectPos then -- place defaultBet if near betObjectId
    local betObjectTile = g_map.getTile(betObjectPos)
    if betObjectTile then
      local betObjectTileTop = betObjectTile:getTopLookThing()
      if betObjectTileTop and betObjectTileTop:getId() == betObjectId then -- if cc hasn't been placed on betObjectId
        local cc = hasCC()
        if cc then -- if found cc in opened backpacks then place them
          g_game.move(cc, betObjectPos, defaultBet)
        else -- if not then try to exchange gold nuggets/dark coins
          local goldNuggets = g_game.findItemInContainers(exchangeIds[1])
          if goldNuggets then -- exchange gold nuggets
            if goldNuggets:getCount() == 100 then
              g_game.move(goldNuggets, {x=65535, y=3, z=0}, 1)
            else
              g_game.use(goldNuggets)
            end
          else -- when out of gold nuggets, exchange dark coins
            local darkCoins = g_game.findItemInContainers(exchangeIds[2])
            if darkCoins then
              g_game.use(darkCoins)
            end
          end
        end
      else -- if cc is placed on betObjectId
        if switchObjectPos then -- use lever if near switch
          local switchObjectTile = g_map.getTile(switchObjectPos)
          if switchObjectTile then
            local switchObjectTileTop = switchObjectTile:getTopLookThing()
            local betObjectTileTop = g_map.getTile(betObjectPos):getTopLookThing()
            if switchObjectTileTop and table.find(switchObjectIds, switchObjectTileTop:getId()) then
              if table.find(possibleBets, betObjectTileTop:getCount()) then
                g_game.use(switchObjectTileTop)
              else
                g_game.move(betObjectTileTop, {x=65535, y=3, z=0}, betObjectTileTop:getCount())
              end
            end
          end
        end
      end
    end
  end
  moveWinToBp(playerPos)
end)

local moneyIds =
{
  3040, -- gold nugget
  3043, -- cc
  11587, -- dark coins
  3035 -- platinum coins
}

macro(500, "Exchange Money", function()
  for i, container in pairs(getContainers()) do
    for j, item in ipairs(container:getItems()) do
      if item:isStackable() and table.find(moneyIds, item:getId()) and item:getCount() == 100 then
        return use(item)
      end
    end
  end
end)
macro(10000, "Anti Kick",  function()
  local dir = player:getDirection()
  turn((dir + 1) % 4)
  turn(dir)
end)
macro(100, "Stack Items", function()
  local containers = g_game.getContainers()
  local toStack = {}
  for index, container in pairs(containers) do
    if not container.lootContainer then -- ignore monster containers
      for i, item in ipairs(container:getItems()) do
        if item:isStackable() and item:getCount() < 100 then
          local stackWith = toStack[item:getId()]
          if stackWith then
            g_game.move(item, stackWith[1], math.min(stackWith[2], item:getCount()))
            return
          end
          toStack[item:getId()] = {container:getSlotPosition(i - 1), 100 - item:getCount()}
        end
      end
    end
  end
end)

macro(10000, "Anti Kick",  function()
  local oldDir = direction()
  turn((oldDir + 1) % 4)
  schedule(1000, function() -- Schedule a function after 1000 milliseconds.
    turn(oldDir)
  end)
end)

openBackpacks = function()
   for _, container in pairs(g_game.getContainers()) do
        g_game.close(container)
   end
    schedule(1000, function()
        bpItem = getBack()
        if bpItem ~= nil then g_game.open(bpItem) end
    end)

    schedule(2000, function()
        local nextContainers = {}
        containers = getContainers()
        for i, container in pairs(g_game.getContainers()) do
            for i, item in ipairs(container:getItems()) do
                if item:isContainer() then
                    table.insert(nextContainers, item)
                end
            end
            if #nextContainers == 0 then return end
            local delay = 1
            for i = 1, #nextContainers do
                schedule(delay, function()
                    g_game.open(nextContainers[i], nil)
                end)
                delay = delay + 250
            end
        end
    end)
end

-- this loads the function straighta way when reloading config
openBackpacks()
